/*
 * The MIT License (MIT)
 *
 * Copyright © 2023 xrv <xrv@live.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.tzld.crawler.etl.util;


import com.tzld.crawler.etl.common.FakeUserAgent;
import com.tzld.crawler.etl.common.enums.ExceptionEnum;
import com.tzld.crawler.etl.common.exception.CommonException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.*;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author ehlxr
 * @since 2023-06-09 15:54.
 */
public class FileUtils {
    private FileUtils() {
    }

    private static final Logger log = LoggerFactory.getLogger(FileUtils.class);

    public static String formatFileSize(long sizeInBytes) {
        if (sizeInBytes < 1024) {
            return sizeInBytes + "B";
        } else if (sizeInBytes < 1024 * 1024) {
            double sizeInKB = sizeInBytes / 1024.0;
            return String.format("%.2fKB", sizeInKB);
        } else {
            double sizeInMB = sizeInBytes / (1024.0 * 1024.0);
            return String.format("%.2fMB", sizeInMB);
        }
    }

    public static String yougetDownload(String yougetPath, String fileUrl, String filePath) throws Exception {
        ProcessBuilder processBuilder = new ProcessBuilder(yougetPath, "-d", fileUrl, "-O", filePath);
        Process process = processBuilder.start();
        process.waitFor();

        // 创建 BufferedReader 以读取进程的输出
        BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
        String line;

        // 创建正则表达式以匹配文件路径
        Pattern pattern = Pattern.compile("Downloading\\s+(\\S*)");

        // 读取并处理输出
        while ((line = reader.readLine()) != null) {
            // System.out.println(line);
            // 如果输出行包含文件路径，打印路径
            Matcher matcher = pattern.matcher(line);
            if (matcher.find()) {
                filePath = matcher.group(1);
                System.out.println("Downloaded file path: " + filePath);
                break;
            }
        }

        return filePath;
    }

    public static void download(String fileUrl, String filePath, boolean useUa) throws Exception {
        download(fileUrl, filePath, useUa, false, null);
    }

    public static void download(String fileUrl, String filePath, boolean useUa, boolean useProxy, Map<String, String> proxyInfo) throws Exception {
        log.info("begin download [{}] to [{}] useUa [{}] useProxy [{}] proxyInfo[{}]", fileUrl, filePath, useUa, useProxy, proxyInfo);
        URL url = new URL(fileUrl);
        HttpURLConnection conn;
        if (useProxy) {
            String proxyUrl = proxyInfo.getOrDefault("url", "");
            int port = Integer.parseInt(proxyInfo.getOrDefault("port", "0"));
            String username = proxyInfo.getOrDefault("username", "");
            String password = proxyInfo.getOrDefault("password", "");
            // 创建代理服务的地址和端口
            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyUrl, port));
            Authenticator authenticator = new Authenticator() {
                @Override
                public PasswordAuthentication getPasswordAuthentication() {
                    return (new PasswordAuthentication(username, password.toCharArray()));
                }
            };
            Authenticator.setDefault(authenticator);
            conn = (HttpURLConnection) url.openConnection(proxy);
        } else {
            conn = (HttpURLConnection) url.openConnection();
        }

        if (useUa) {
            conn.setRequestProperty("User-Agent", FakeUserAgent.getRandomUserAgent());
        }
        if (conn.getResponseCode() == HttpURLConnection.HTTP_FORBIDDEN) {
            throw new CommonException(ExceptionEnum.URL_FORBIDDEN);
        }
        conn.setConnectTimeout(5000);
        conn.setReadTimeout(5000);
        log.info("download file size is {} of url [{}]", formatFileSize(conn.getContentLength()), fileUrl);

        InputStream inputStream = conn.getInputStream();
        FileOutputStream outputStream = new FileOutputStream(filePath);
        byte[] buffer = new byte[4096];
        int len;
        while ((len = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, len);
        }
        inputStream.close();
        outputStream.close();
        log.info("downloaded successfully [{}] to [{}]", fileUrl, filePath);
    }

    public static void downloadForGZH(String fileUrl, String filePath, boolean useUa, boolean useProxy, Map<String, String> proxyInfo) throws Exception {
        log.info("begin download [{}] to [{}] useUa [{}] useProxy [{}] proxyInfo[{}]", fileUrl, filePath, useUa, useProxy, proxyInfo);
        SSLUtils.ignoreSsl();
        URL url = new URL(fileUrl);
        HttpURLConnection conn;
        if (useProxy) {
            String proxyUrl = proxyInfo.getOrDefault("url", "");
            int port = Integer.parseInt(proxyInfo.getOrDefault("port", "0"));
            String username = proxyInfo.getOrDefault("username", "");
            String password = proxyInfo.getOrDefault("password", "");
            // 创建代理服务的地址和端口
            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyUrl, port));
            Authenticator authenticator = new Authenticator() {
                @Override
                public PasswordAuthentication getPasswordAuthentication() {
                    return (new PasswordAuthentication(username, password.toCharArray()));
                }
            };
            Authenticator.setDefault(authenticator);
            conn = (HttpURLConnection) url.openConnection(proxy);
        } else {
            conn = (HttpURLConnection) url.openConnection();
        }

        if (useUa) {
            conn.setRequestProperty("User-Agent", FakeUserAgent.getRandomUserAgent());
        }

        conn.setRequestProperty("Accept", "*/*");
        conn.setRequestProperty("Accept-Encoding", "identity;q=1, *;q=0");
        conn.setRequestProperty("Accept-Language", "zh-CN,zh;q=0.9");
        conn.setRequestProperty("Host", "mpvideo.qpic.cn");
        conn.setRequestProperty("Origin", "https://mp.weixin.qq.com");
        conn.setRequestProperty("Referer", "https://mp.weixin.qq.com");

        if (conn.getResponseCode() == HttpURLConnection.HTTP_FORBIDDEN) {
            throw new CommonException(ExceptionEnum.URL_FORBIDDEN);
        }
        conn.setConnectTimeout(5000);
        conn.setReadTimeout(5000);
        log.info("download file size is {} of url [{}]", formatFileSize(conn.getContentLength()), fileUrl);

        InputStream inputStream = conn.getInputStream();
        FileOutputStream outputStream = new FileOutputStream(filePath);
        byte[] buffer = new byte[4096];
        int len;
        while ((len = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, len);
        }
        inputStream.close();
        outputStream.close();
        log.info("downloaded successfully [{}] to [{}]", fileUrl, filePath);
    }

    public static void main(String[] args) throws Exception {
        // try {
        System.out.println("https://api-hl.huoshan.com/hotsoon/item/video/_source/?video_id=v0300fg10000ckmbrbbc77uc3nq19840&line=0&app_id=0&vquality=normal&watermark=0&long_video=0&sf=4&ts=1697528496&item_id=7290410334844718376".replace("https://api-hl.huoshan.com", "http://api-hl.huoshan.com"));
        download("http://api-hl.huoshan.com/hotsoon/item/video/_source/?video_id=v0d00fg10000ckjb1bbc77u340cpgelg&line=0&app_id=0&vquality=normal&watermark=0&long_video=0&sf=4&ts=1697529427&item_id=7288707446217002255", "/Users/dingyunpeng/Downloads/" + System.currentTimeMillis(), true);
        //
        // } catch (Exception e) {
        //     e.printStackTrace();
        // }
        // String s = "/Users/ehlxr/Downloads/" + System.currentTimeMillis();
        // String filePath = yougetDownload("/Users/ehlxr/Desktop/you-get-0.4.1650/you-get", "http://mpvideo.qpic.cn/0bc3amabsaaa7mamoixezzrfaa6ddebqagia.f10002.mp4?dis_k=683cf1375dd8cd051ebc48c9861dbf65&dis_t=1694751341&play_scene=10120&auth_info=JO3Lo+01NkNXiZupx3kqLiQ0OB8XZmsYYXs4QSNiQDMCV3IxCDlrLDU9D2AcXT0ULg==&auth_key=33f6a5927b824d086369a12ff4a7a635&vid=wxv_2509438417202888705&format_id=10002&support_redirect=0&mmversion=false", s);
        //
        // File file = new File(filePath);
        // System.out.println(file.exists());

        // try {
        //     // 创建和启动进程
        //     ProcessBuilder pb = new ProcessBuilder("/Users/ehlxr/Desktop/you-get-0.4.1650/you-get", "-O", "/Users/ehlxr/Downloads/" + System.currentTimeMillis(), "http://mpvideo.qpic.cn/0bc3zyaagaaaieajowybojsvbtwdaphaaaya.f10002.mp4?dis_k=afa8996b6f4aac67ff2d6b3b7abaa4b4&dis_t=1694751571&play_scene=10120&auth_info=WsS8pdtVOTQL3MuqxRszQlg3FBNoCCQ4PQATPSseNWV8Sz4/BF1kW2kwH14QOSR4Ug==&auth_key=d33c33aa66ca8bd8a05204709a5f92b1&vid=wxv_3103954619094892547&format_id=10002&support_redirect=0&mmversion=false");
        //     pb.redirectErrorStream(true);
        //     Process process = pb.start();
        //
        //     // 创建 BufferedReader 以读取进程的输出
        //     BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
        //     String line;
        //
        //     // 创建正则表达式以匹配文件路径
        //     Pattern pattern = Pattern.compile("Downloading (.*)", Pattern.CASE_INSENSITIVE);
        //
        //     // 读取并处理输出
        //     while ((line = reader.readLine()) != null) {
        //         // 如果输出行包含文件路径，打印路径
        //         Matcher matcher = pattern.matcher(line);
        //         if (matcher.find()) {
        //             System.out.println("Downloaded file path: " + matcher.group(1));
        //         }
        //     }
        //
        //     // 等待进程结束
        //     process.waitFor();
        // } catch (Exception e) {
        //     e.printStackTrace();
        // }
    }
}
